<?php
/**
 * con4gis - the gis-kit
 *
 * @version   php 5
 * @package   con4gis
 * @author    con4gis contributors (see "authors.txt")
 * @license   GNU/LGPL http://opensource.org/licenses/lgpl-3.0.html
 * @copyright Küstenschmiede GmbH Software & Design 2011 - 2016.
 * @link      https://www.kuestenschmiede.de
 */

namespace con4gis\MapsBundle\Classes;


use con4gis\MapsBundle\Entity\BaseLayer;

class BaseLayerParser
{
    public function parseBaseLayer(BaseLayer $baseLayer)
    {
        $stringClass = $GLOBALS['con4gis']['stringClass'];
        $arrBaseLayer = array();

        $arrBaseLayer['id'] = $baseLayer->getId();
        $arrBaseLayer['name'] =  \Contao\Controller::replaceInsertTags($stringClass::decodeEntities($baseLayer->getDisplayName() ?: $baseLayer->getName()));

        $arrBaseLayer['provider'] = $baseLayer->getProvider();
        switch ($baseLayer->getProvider()) {
            case 'osm':
                $arrBaseLayer = array_merge($arrBaseLayer, $this->parseOsmLayer($baseLayer));
                break;
            case 'mapbox':
                $arrBaseLayer = array_merge($arrBaseLayer, $this->parseMapboxLayer($baseLayer));
                break;
            case 'klokan':
                $arrBaseLayer = array_merge($arrBaseLayer, $this->parseKlokanLayer($baseLayer));
                break;
            case 'here':
                $arrBaseLayer = array_merge($arrBaseLayer, $this->parseHereLayer($baseLayer));
                break;
            case 'thunder':
                $arrBaseLayer = array_merge($arrBaseLayer, $this->parseThunderLayer($baseLayer));
                break;
            case 'google':
                $arrBaseLayer = array_merge($arrBaseLayer, $this->parseGoogleLayer($baseLayer));
                break;
            case 'bing':
                $arrBaseLayer = array_merge($arrBaseLayer, $this->parseBingLayer($baseLayer));
                break;
            case 'wms':
                $arrBaseLayer = array_merge($arrBaseLayer, $this->parseWmsLayer($baseLayer));
                break;
            case 'owm':
                $arrBaseLayer = array_merge($arrBaseLayer, $this->parseOwmLayer($baseLayer));
                break;
            default:
                die('This should not have happened!');
        }
        if (!empty($baseLayer->getAttribution())) {
            $arrBaseLayer['attribution'] = $baseLayer->getAttribution();
        }
        if (!empty($baseLayer->getMinzoomlevel())) {
            $arrBaseLayer['minZoom'] = $baseLayer->getMinzoomlevel();
        }
        if (!empty($baseLayer->getMaxzoomlevel())) {
            $arrBaseLayer['maxZoom'] = $baseLayer->getMaxzoomlevel();
        }
        if (!empty($baseLayer->getSorting())) {
            $arrBaseLayer['sort'] = $baseLayer->getSorting();
        }
        $arrBaseLayer['cesium']= $baseLayer->getCesium();
        return $arrBaseLayer;
    }

    private function parseOsmLayer(BaseLayer $baseLayer)
    {
        $arrBaseLayer = [];
        $arrBaseLayer['style'] = $baseLayer->getOsmStyle();
        if (!empty($baseLayer->getOsmKeyname())) {
            $arrBaseLayer['apiKey'] = $baseLayer->getOsmKeyname();
        }
        // custom?
        if ($arrBaseLayer['style'] == 'osm_custom') {
            if (!empty($baseLayer->getOsmStyleUrl1()) && empty($baseLayer->getOsmStyleUrl2())) {
                $arrBaseLayer['url'] = str_replace("$", "", $baseLayer->getOsmStyleUrl1());
            } else {
                if (!empty($baseLayer->getOsmStyleUrl1())) {
                    $arrBaseLayer['urls'][] = str_replace("$", "", $baseLayer->getOsmStyleUrl1());
                }
                if (!empty($baseLayer->getOsmStyleUrl2())) {
                    $arrBaseLayer['urls'][] = str_replace("$", "", $baseLayer->getOsmStyleUrl2());
                }
                if (!empty($baseLayer->getOsmStyleUrl3())) {
                    $arrBaseLayer['urls'][] = str_replace("$", "", $baseLayer->getOsmStyleUrl3());
                }
                if (!empty($baseLayer->getOsmStyleUrl4())) {
                    $arrBaseLayer['urls'][] = str_replace("$", "", $baseLayer->getOsmStyleUrl4());
                }
            }
        }

        return $arrBaseLayer;
    }

    private function parseMapboxLayer(BaseLayer $baseLayer)
    {
        $arrBaseLayer = [];
        $arrBaseLayer['url'] = 'https://api.mapbox.com/styles/v1/';
        $arrBaseLayer['url_classic'] = 'https://api.tiles.mapbox.com/v4/';
        $arrBaseLayer['mapbox_type'] = $baseLayer->getMapboxType();
        if($baseLayer->getHideInBe())
        {
            $arrBaseLayer['hide_in_be'] = $baseLayer->getHideInBe();
        }
        else
        {
            $arrBaseLayer['app_id'] = $baseLayer->getAppId();
            $arrBaseLayer['api_key'] = $baseLayer->getApiKey();
        }
        return $arrBaseLayer;
    }

    private function parseKlokanLayer(BaseLayer $baseLayer)
    {
        $arrBaseLayer = [];
        $arrBaseLayer['url'] = $baseLayer->getUrl();
        $klokan_type = $baseLayer->getKlokanType();
        if ($klokan_type != 'OpenMapTiles') {
            $klokan_type = 'TileHosting';
        }
        $arrBaseLayer['style'] = $baseLayer->getKlokanType();
        $arrBaseLayer['klokan_type'] = $klokan_type;
        $arrBaseLayer['app_id'] = $baseLayer->getAppId();
        $arrBaseLayer['api_key'] = $baseLayer->getApiKey();
        return $arrBaseLayer;
    }

    private function parseHereLayer(BaseLayer $baseLayer)
    {
        $arrBaseLayer = [];
        $arrBaseLayer['style'] = $baseLayer->getHereType();
        $arrBaseLayer['here_type'] = 'HERE';
        if($baseLayer->getHideInBe())
        {
            $arrBaseLayer['hide_in_be'] = $baseLayer->getHideInBe();
        }
        else
        {
            $arrBaseLayer['app_id'] = $baseLayer->getAppId();
            $arrBaseLayer['api_key'] = $baseLayer->getApiKey();
        }
        return $arrBaseLayer;
    }

    private function parseThunderLayer(BaseLayer $baseLayer)
    {
        $arrBaseLayer = [];
        $arrBaseLayer['style'] = $baseLayer->getThunderforestType();
        $arrBaseLayer['thunderforest_type'] = 'Thunderforest';
        if($baseLayer->getHideInBe())
        {
            $arrBaseLayer['hide_in_be'] = $baseLayer->getHideInBe();
        }
        else
        {
            $arrBaseLayer['api_key'] = $baseLayer->getApiKey();
        }
        return $arrBaseLayer;
    }

    private function parseGoogleLayer(BaseLayer $baseLayer)
    {
        $arrBaseLayer = [];
        $arrBaseLayer['style'] = $baseLayer->getGoogleStyle();
        return $arrBaseLayer;
    }

    private function parseBingLayer(BaseLayer $baseLayer)
    {
        $arrBaseLayer = [];
        $arrBaseLayer['style'] = $baseLayer->getBingStyle();
        if (!empty($baseLayer->bing_key)) {
            $arrBaseLayer['apiKey'] = $baseLayer->getBingKey();
        }
        return $arrBaseLayer;
    }

    private function parseWmsLayer(BaseLayer $baseLayer)
    {
        $arrBaseLayer = [];
        $arrBaseLayer['url'] = $baseLayer->getWmsUrl();

        $arrBaseLayer['params'] = array();
        $arrBaseLayer['params']['layers'] = $baseLayer->getWmsParamsLayers();
        $arrBaseLayer['params']['version'] = $baseLayer->getWmsParamsVersion();
        $arrBaseLayer['params']['format'] = $baseLayer->getWmsParamsFormat();
        $arrBaseLayer['params']['transparent'] = $baseLayer->getWmsParamsTransparent() ? true : false;
        $arrBaseLayer['params']['srs'] = $baseLayer->getWmsParamsSrs();

        $arrBaseLayer['gutter'] = $baseLayer->getWmsGutter();
        return $arrBaseLayer;
    }

    private function parseOwmLayer(BaseLayer $baseLayer)
    {
        $arrBaseLayer = [];
        // TODO feld existiert nicht, richtigen Port finden und Url wieder einbauen!
//                $arrBaseLayer['url'] = 'http://maps.owm.io:'.$baseLayer->api_port.'/';

        $arrBaseLayer['app_id'] = $baseLayer->getAppId();
        $arrBaseLayer['api_key'] = $baseLayer->getApiKey();
        return $arrBaseLayer;
    }
}