<?php
/**
 * con4gis - the gis-kit
 *
 * @version   php 5
 * @package   con4gis
 * @author    con4gis contributors (see "authors.txt")
 * @license   GNU/LGPL http://opensource.org/licenses/lgpl-3.0.html
 * @copyright Küstenschmiede GmbH Software & Design 2011 - 2016.
 * @link      https://www.kuestenschmiede.de
 */

namespace con4gis\MapsBundle\Classes;


use con4gis\MapsBundle\Entity\Layer;
use con4gis\MapsBundle\Repository\LayerRepository;
use con4gis\MapsBundle\Resources\contao\modules\api\LayerContentApi;
use Contao\Database;
use Doctrine\ORM\EntityRepository;

class LayerParser
{
    protected $arrLayers = array();
    protected $arrConfig = array();
    protected $arrReassignedLayer = array();

    /**
     * @var LayerRepository
     */
    protected $layerRepository = null;

    /** @var LayerContentParser */
    protected $contentParser = null;

    /**
     * LayerParser constructor.
     * @param EntityRepository $layerRepository
     * @param LayerContentParser $contentParser
     */
    public function __construct(EntityRepository $layerRepository, LayerContentParser $contentParser)
    {
        if ($layerRepository instanceof LayerRepository) {
            $this->layerRepository = $layerRepository;
        }
        $this->contentParser = $contentParser;
    }

    /**
     * Determines the request method and selects the appropriate data result.
     *
     * @param  array $arrInput Fragments from request uri
     * @return mixed           JSON data
     */


    public function getLayers($intParentId)
    {
        \FrontendUser::getInstance()->authenticate();

        $arrLayers = $this->getLayerList($intParentId);

        if(sizeof($this->arrReassignedLayer) > 0)
        {
            foreach ($arrLayers as $index=>$layer)
            {
                $strLayerName =  \Contao\Controller::replaceInsertTags($layer['name']);
                if (array_key_exists($strLayerName, $this->arrReassignedLayer))
                {
                    foreach ($this->arrReassignedLayer[$strLayerName] as $reassignedLayer)
                    {
                        if (is_array($arrLayers[$index]['content'])) {
                            $arrLayers[$index]['content'][] = $reassignedLayer;
                        } else {
                            $arrLayers[$index]['content'] = array($reassignedLayer);

                        }
                    }
                }

                if ($layer['hasChilds'])
                {
                    foreach ($layer['childs'] as $childIndex=>$childLayer)
                    {
                        $strChildLayerName =  \Contao\Controller::replaceInsertTags($childLayer['name']);
                        if (array_key_exists($strChildLayerName, $this->arrReassignedLayer))
                        {
                            foreach ($this->arrReassignedLayer[$strChildLayerName] as $reassignedLayer)
                            {
                                if (is_array($arrLayers[$index]['childs'][$childIndex]['content'])) {
                                    $arrLayers[$index]['childs'][$childIndex]['content'][] = $reassignedLayer;
                                } else {
                                    $arrLayers[$index]['childs'][$childIndex]['content'] = array($reassignedLayer);
                                }
                            }
                        }
                    }
                }
            }
        }

        $this->arrConfig['countAll'] = sizeof($arrLayers);
        $return = array(
            'config' => $this->arrConfig,
            'layer' => $arrLayers,
            'foreignLayers' => $this->checkAndFetchMissingLinkedLayers($arrLayers)
        );
        foreach($return['layer'] as $key => $layer)
        {
            /** @var Layer $objLayer */
            $objLayer = $this->layerRepository->findOneBy(['id' => $layer['id']]);
            if ($objLayer->)
            {
                $return['layer'][$key] = $this->forceChildsInContent($layer);
            }
        }
        $return['layer'] = $this->checkAndReassignFrontendLayers($return['layer']);
        $db = Database::getInstance();
        $db->prepare("DELETE FROM tl_c4g_map_layer_content")->execute();
        foreach($return['layer'] as $key => $layer){
            $return['layer'][$key] = $this->saveLayerContent($layer);
        }


        return $return;
    }

    /**
     * Returns the layer structure for the map.
     *
     * @param int $id
     */
    protected function getLayerList($intId, $blnIsSubLayer = false)
    {
        $arrLayer = array();
        $arrLinkData = array();

        if (!$blnIsSubLayer) {
            // Find the requested map
            /** @var Layer $objMap */
            $objMap = $this->layerRepository->findOneBy(['id' => $intId]);
            // Only return map entries
            if ($objMap == null || !$objMap->getisMap()) {
                // TODO return 404 HTTP
                HttpResultHelper::NotFound();
            }
        }

        // Get all layers on the map
        $arrLayers = $this->layerRepository->findPublishedByPid($intId);

        if ($objMap) {
            // append map itself as structure element
            if ($objMap->getLocationType() != "none") {
                $mapLayer = $this->parseLayer($objMap);
                unset($mapLayer['raw']);
                $arrLayer[] = $mapLayer;
            }
        }
        /** @var Layer $layer */
        foreach ($arrLayers as $layer) {
            // do not return protected layers, to users without permission
            if ($layer->getProtectElement()) {
                if (\FrontendUser::getInstance()->authenticate() && !empty($layer->getPermittedGroups())) {
                    // get groups data from tl_member table
                    $userGroups = \FrontendUser::getInstance()->getData()['groups'];
                    $permittedGroups = deserialize($layer->getPermittedGroups());
                    if (sizeof(array_intersect($userGroups, $permittedGroups)) < 1) {
                        continue;
                    }
                } else {
                    continue;
                }
            }

            if ($arrGetLayerData = $this->parseLayer($layer)) {
                $arrLayerData = $arrGetLayerData;//$this->parseLayer($objLayers);
                $arrLayerData['cssClass'] = $layer->getCssClass();

                if ($childLayerList = $this->getLayerList($arrLayerData['id'], true)) {
                    $arrLayerData['hasChilds'] = true;
                    $arrLayerData['childsCount'] = sizeof($childLayerList);
                    $arrLayerData['childs'] = $childLayerList;
                } else if ($layer->getLocationType() == 'link') {
                    // link handling
                    if ($childLayerList = $this->getLayerList($arrLayerData['link_id'], true)) {
                        // duplicate children for the link
                        foreach($childLayerList as $key=>$child)
                        {
                            // generate new unique ids
                            $childId = $childLayerList[$key]['id'];
                            $childLayerList[$key]['link_id'] = $childId;
                            $childLayerList[$key]['id'] = uniqid();
                            $childLayerList[$key]['pid'] = $arrLayerData['id'];
                            $childLayerList[$key]['childs'] = $this->getLinkedChilds($childLayerList[$key]);
                            if ($childLayerList[$key]['childs'] && sizeof($childLayerList[$key]['childs']) > 0) {
                                $childLayerList[$key]['hasChilds'] = true;
                                $childLayerList[$key]['childCount'] = sizeof($childLayerList[$key]['childs']);
                                foreach($childLayerList[$key]['childs'] as $index=>$item)
                                {
                                    $childLayerList[$key]['childs'][$index]['hide'] = $layer->getDataHidelayer();
                                }
                            }
                        }
                        $arrLayerData['hasChilds'] = true;
                        $arrLayerData['childsCount'] = sizeof($childLayerList);
                        $arrLayerData['childs'] = $childLayerList;
                    }
                } else if ($layer->getLocationType() == 'folder') {
                    $folder = $this->getFolder($layer);
                    $noneFolder = false;
                    if ($folder != '') {
                        if ($noneFolder) {
                            $dict = $this->contentParser->getFolderData($layer, 1);
                            foreach ($dict as $child) {
                                $arrLayer[] = $child;
                            }
                            continue;
                        } else {
                            $dict = $this->contentParser->getFolderData($layer);

                            $arrLayerData['hasChilds'] = true;
                            $arrLayerData['childsCount'] = count($dict);
                            $arrLayerData['childs'] = $dict;

                            unset($arrLayerData['raw']);
                            $arrLayer[] = $arrLayerData;
                            continue;
                        }
                    }
                }
                $arrLayerData['childs'] = $this->setChildHide($arrLayerData['childs'], $layer);

                // HOOK: add custom logic
                // TODO change to symfony events, but all connected bundles have to be changed as well!
                if (isset($GLOBALS['TL_HOOKS']['c4gAddLocationsParent']) && is_array($GLOBALS['TL_HOOKS']['c4gAddLocationsParent']))
                {
                    foreach ($GLOBALS['TL_HOOKS']['c4gAddLocationsParent'] as $callback)
                    {
                        $str_class = $callback[0];
                        $str_function = $callback[1];
                        if ($str_class && $str_function) {
                            $callClass = new $str_class();
                            $arrData = $callClass->$str_function($arrLayerData['pid'], $arrLayerData, $this);
                            if ($arrData && is_array($arrData) && sizeof($arrData)>0)
                            {
                                $arrLayerData = $arrData;
                            }
                        }
                    }
                }
                unset($arrLayerData['raw']);
                $arrLayer[] = $arrLayerData;
            }
        }
        return $arrLayer;
    }

    /**
     * Private function to set the hide property of childs correctly.
     * @param $childList
     * @param $parentLayer
     * @return array
     */
    private function setChildHide($childList, Layer $parentLayer)
    {
        $newChildList = array();
        if ($childList && $parentLayer) {
            foreach($childList as $index=>$child)
            {
                if ($parentLayer->getDataHidelayer()) {
                    $child['hide'] = $parentLayer->getDataHidelayer();
                    if ($child['hasChilds']) {
                        $child['childs'] = $this->setChildHide($child['childs'], $parentLayer);
                    }
                }
                $newChildList[$index] = $child;
            }
        }
        return $newChildList;
    }

    private function getLinkedChilds($layer)
    {
        $childs = array();
        $arrLayers = $this->layerRepository->findPublishedByPid($layer['link_id']);
        if ($arrLayers) {
            /** @var Layer $layer */
            foreach($arrLayers as $layer) {
                $child = array();
                $child['link_id'] = $layer['id'];
                $child['name'] = \Contao\Controller::replaceInsertTags($layer['name']);
                $child['id'] = uniqid();
                $child['pid'] = $layer['id'];
                $child['display'] = $layer['display'];
                $child['content'] = $this->getContentForType($layer);
                $child['type'] = "link";
                $child['childs'] = $this->getLinkedChilds($child);
                if ($child['childs'] && sizeof($child['childs']) > 0) {
                    $child['hasChilds'] = true;
                    $child['childCount'] = sizeof($child['childs']);
                }
                $childs[] = $child;
            }
            return $childs;
        } else {
            return array();
        }
    }

    private function getFolder(Layer $layer)
    {
        $strUrl = "";
        if ($layer->getDataFolder())
        {
            if (\Validator::isUuid($layer->getDataFolder()))
            {
                // add ressource folder
                $objFolder = \FilesModel::findByUuid($layer->getDataFolder());
                if ($objFolder !== null)
                {
                    $strUrl = $objFolder->path;
                }
            }
        }
        return $strUrl;
    }

    /**
     * Summary of parseLayer
     *
     * @param mixed $objLayer
     * @return array
     */
    protected function parseLayer(Layer $layer)
    {
        $stringClass = $GLOBALS['con4gis']['stringClass'];
        $arrLayerData = array();
        $arrLayerData['id'] = $layer->getId();
        $arrLayerData['pid'] = $layer->getPid();
        $arrLayerData['name'] =  \Contao\Controller::replaceInsertTags($stringClass::decodeEntities($layer->getName()));
        $arrLayerData['zoom_locations'] = $layer->getZoomLocations();
        $arrLayerData['async_content'] = $layer->getAsyncContent();

        // check parent hide status
        /** @var Layer $parentLayer */
        $parentLayer = $this->layerRepository->findOneById($layer->getPid());
        if (($parentLayer) && (!$parentLayer->getIsMap() && $parentLayer->getDataHidelayer())) {
            $arrLayerData['hide'] = $parentLayer->getDataHidelayer();
        } else {
            $arrLayerData['hide'] = $layer->getDataHidelayer();
        }

        if ($layer->getLocMinzoom() > 0 || $layer->getLocMaxzoom() > 0)
        {
            $arrLayerData['zoom'] = array(
                'min' => $layer->getLocMinzoom(),
                'max' => $layer->getLocMaxzoom(),
                'onclick_to' => $layer->getLocOnclickZoomto()
            );
        }

        if ($layer->getDataLayername())
        {
            $arrLayerData['name'] = \Contao\Controller::replaceInsertTags($stringClass::decodeEntities($layer->getDataLayername()));
            $arrLayerData['display'] = true;
            $arrLayerData['hide_child'] = $layer->getHideChild();
        }
        else
        {
            $arrLayerData['display'] = false;
        }

        // hide when element is rendered in starboard tab
        if ($layer->getHideWhenInTab() == 1) {
            $arrLayerData['hide_when_in_tab'] = true;
        }

        $arrLayerData['type'] = $layer->getLocationType();
        if ($layer->getLocationType() === 'link') {
            $arrLayerData['link_id'] = $layer->getLinkId();
            /** @var Layer $linkedLayer */
            $linkedLayer = $this->layerRepository->findOneById($layer->getLinkId());
            if ($linkedLayer) {
                // check if linked element is overpass request and assign correct content values
                if ($linkedLayer->getLocationType() == "overpass") {
                    $arrLayerData['content'] = $this->getContentForType($linkedLayer);
                } else {
                    $arrLayerData['content'] = $this->getContentForType($layer);
                    $arrLayerData['hide'] = $layer->getDataHidelayer();
                }
                // set zooms of links
                if ($linkedLayer->getLocMinzoom() > 0 || $linkedLayer->getLocMaxzoom() > 0)
                {
                    $arrLayerData['zoom'] = array(
                        'min' => $linkedLayer->getLocMinzoom(),
                        'max' => $linkedLayer->getLocMaxzoom(),
                        'onclick_to' => $linkedLayer->getLocOnclickZoomto()
                    );
                }
            }

        } else {
            $arrLayerData['content'] = $this->getContentForType($layer);
        }
        if ($layer->getLocationType() === 'startab') {
            $arrLayerData['awesomeicon'] = $layer->getAwesomeicon();
        }
        $arrLayerData['raw'] = $layer;

        return $arrLayerData;
    }

    /**
     * @param $objLayer
     * @return array|bool
     */
    protected function getContentForType(Layer $layer)
    {
        switch ($layer->getLocationType())
        {
            // TODO: make Hook instead and let con4gis-Forum handle this?
            case "c4gForum":
                if ($layer->getForumReassignLayer() && $layer->getForumReassignLayer()=="THREAD") {
                    $arrReassignedRawLayer = $this->contentParser->getLayerData($layer->getId());

                    //loop over all forum layers
                    foreach ($arrReassignedRawLayer as $index=>$reassignedLayer)
                    {
                        $found = false;
                        $strReassignName = $reassignedLayer['threadName'];
                        $layer = $this->arrReassignedLayer[$strReassignName];
                        if ($layer && is_array($layer)) {
                            $this->arrReassignedLayer[$strReassignName][] = $reassignedLayer;
                            $found = true;
                        } else if (!$layer) {
                            // lookup in child elements if the layer is there
                            foreach ($this->arrReassignedLayer as $index_child=>$reassignedLayer_child)
                            {
                                $layer_child = $reassignedLayer_child[$strReassignName];
                                if ($layer_child && is_array($layer_child)) {
                                    // found layer
                                    $reassignedLayer_child[$strReassignName][] = $reassignedLayer;
                                    $found = true;
                                    break;
                                }
                            }
                        }

                        if (!$found) {
                            //since the layer was not found, insert it at first level
                            $this->arrReassignedLayer[$strReassignName] = array();
                            $this->arrReassignedLayer[$strReassignName][] = $reassignedLayer;
                        }
                    }

                    return false;
                } else {
                    return $this->contentParser->getLayerData($layer->getId());
                }
            // same function call, so fallthrough
            case "link":
            case "overpass":
            case "gpx":
            case "kml":
            case "osm":
            case "table":
            case "single":
                return $this->contentParser->getLayerData($layer->getId());
//            case "folder":
//                return $objLayerContentApi->getFolderDataPublic($objLayer);
            default:
                if (!$layer->getDataHidelayer()) {
                    return $this->contentParser->getLayerData($layer->getId());
                }
                return false;
        }
    }

    protected function saveLayerContent($layer)
    {
        if($layer['childs']){
            foreach($layer['childs'] as $key => $child)
            {
                $layer['childs'][$key] = $this->saveLayerContent($child);
            }
        }
        if($layer['async_content'] == 1) {
            if ($layer['content']) {
                foreach ($layer['content'] as $key => $content) {
                    if (!$content['data'] || !$content['data']['geometry'] || !$content['data']['geometry']['coordinates'] || count($content['data']['geometry']['coordinates']) != 2) continue;
                    $set['pid'] = $layer['id'];
                    $set['type'] = $content['type'];
                    $set['format'] = $content['format'];
                    $set['origType'] = $content['origType'];
                    $set['locStyle'] = $content['locationStyle'];
                    $set['datatype'] = $content['data']['type'];
                    $set['geotype'] = $content['data']['geometry']['type'];
                    $set['geox'] = $content['data']['geometry']['coordinates'][0];
                    $set['geoy'] = $content['data']['geometry']['coordinates'][1];
                    $set['projection'] = $content['data']['properties']['projection'];
                    $set['popup_content'] = $content['data']['properties']['popup']['content'];
                    $set['popup_routing_link'] = $content['data']['properties']['popup']['routing_link'];
                    $set['popup_async'] = $content['data']['properties']['popup']['async'];
                    $set['tooltip'] = $content['data']['properties']['tooltip'];
                    $set['tooltip_length'] = $content['data']['properties']['tooltip_length'];
                    $set['label'] = $content['data']['properties']['label'];
                    $set['loc_linkurl'] = $content['data']['properties']['loc_linkurl'];
                    $set['hover_location'] = $content['data']['properties']['hover_location'];
                    $set['hover_style'] = $content['data']['properties']['hover_style'];
                    $set['cluster_fillcolor'] = $content['cluster_fillcolor'];
                    $set['cluster_distance'] = $content['cluster_distance'];
                    $set['cluster_fontcolor'] = $content['cluster_fontcolor'];
                    $set['cluster_zoom'] = $content['cluster_zoom'];
                    $set['cluster_popup'] = $content['cluster_popup'];
                    if($content['settings']['cluster']){
                        $layer['cluster'] = $content['settings']['cluster'];
                    }
                    $db = Database::getInstance();
                    $db->prepare("INSERT INTO tl_c4g_map_layer_content %s")->set($set)->execute();
                    unset($layer['content'][$key]);
                }
            }
        }

        return $layer;
    }
    protected function forceChildsInContent($layer)
    {
        $arrChilds =[];
        foreach($layer['childs'] as $key => $child)
        {
            if($child['childs'] && $child['childs'][0]['content'] && $child['childs'][0]['content'] != []) {
                $arrContent = [];
                foreach ($child['childs'] as $children) {
                    $arrContent[] = $children['content'][0];
                }
                $child['content'] = $arrContent;
                unset($child['childs']);
                unset($child['childsCount']);
                $child['hasChilds'] = false;
                $arrChilds[] = $child;
            }
            else {
                $child['childs'] = $this->forceChildsInContent($layer['childs'][0]);
            }
        }
        $layer['childs'] = $arrChilds;
        return $layer;
    }

    protected function checkAndFetchMissingLinkedLayers($layers)
    {
        $missingLayers = array();

        $layerIdList = array();
        $this->setLayerIdList($layers, $layerIdList);

        foreach ($layerIdList as $id => $exists) {
            if (!$exists) {
                $fetchedLayer = $this->layerRepository->findOneById($id);
                if (!$fetchedLayer) {
                    continue;
                }
                // do not return protected layers, to users without permission
                if ($fetchedLayer->getProtectElement()) {
                    if (\FrontendUser::getInstance()->authenticate() && !empty($fetchedLayer->getPermittedGroups())) {
                        // get groups data from tl_member table
                        $userGroups = \FrontendUser::getInstance()->getData()['groups'];
                        $permittedGroups = deserialize($fetchedLayer->getPermittedGroups());
                        if (sizeof(array_intersect($userGroups, $permittedGroups)) < 1) {
                            continue;
                        }
                    } else {
                        continue;
                    }
                }
                if ($layerData = $this->parseLayer($fetchedLayer)) {

                    if ($childLayerList = $this->getLayerList($layerData['id'], true)) {
                        $layerData['hasChilds'] = true;
                        $layerData['childsCount'] = sizeof($childLayerList);
                        $layerData['childs'] = $childLayerList;
                    }

                    // HOOK: add custom logic
                    // TODO change to symfony events, but all connected bundles have to be changed as well!
                    if (isset($GLOBALS['TL_HOOKS']['c4gAddLocationsParent']) && is_array($GLOBALS['TL_HOOKS']['c4gAddLocationsParent']))
                    {
                        foreach ($GLOBALS['TL_HOOKS']['c4gAddLocationsParent'] as $callback)
                        {
                            $str_class = $callback[0];
                            $str_function = $callback[1];
                            if ($str_class && $str_function) {
                                $this->import($str_class);
                                $arrData = $this->$str_class->$str_function($layerData['pid'], $layerData, $this);
                                if ($arrData && is_array($arrData) && sizeof($arrData)>0)
                                {
                                    $layerData = $arrData;
                                }
                            }
                        }
                    }
                    unset($layerData['raw']);
                    $missingLayers[$layerData['id']] = $layerData;
                }
            }
        }

        return $missingLayers;
    }

    protected function setLayerIdList($layers, &$idList)
    {
        foreach ($layers as $layer) {
            $idList[$layer['id']] = true;
            if ($layer['link_id'] && !array_key_exists($layer['link_id'], $idList)) {
                $idList[$layer['link_id']] = false;
            }
            if ($layer['hasChilds']) {
                $this->setLayerIdList($layer['childs'], $idList);
            }
        }
    }



    private function checkAndReassignFrontendLayers(&$arrLayers) {
        foreach($arrLayers as $index => $layer) {
            if ($layer['type'] == "poim") {
                $objLayer = $this->layerRepository->findOneById($layer['id']);
                if ($objLayer && $objLayer->poim_reassign_layer) {
                    foreach($layer['childs'] as $grpKey=> $poiGroup) {
                        $overpassLayers = &$this->searchOverpassLayerByName($poiGroup['name'], $arrLayers);
                        if (count($overpassLayers) > 0) {
                            foreach($overpassLayers as &$overpassLayer) {
                                $overpassLayer['childs'] = array();
                                $overpassLayer['childsCount'] = 0;
                                $poiGroup['pid'] = $overpassLayer['id'];
                                $overpassLayer['childs'][] = $poiGroup;
                                $overpassLayer['hasChilds'] = true;
                            }
                            unset ($layer['childs'][$grpKey]);
                            $layer['childsCount']--;
                        }
                    }
                    $arrLayers[$index]['display'] = false;
                }
            }
        }
        return $arrLayers;
    }

    private function &searchOverpassLayerByName($name, &$layers) {
        $return = array();
        foreach($layers as $key => &$layer) {
            if ($layer['name'] == $name && $layer['type'] == 'overpass') {
                $return[] = &$layer;
            }
            if ($layer['hasChilds']) {
                $recursiveResult = &$this->searchOverpassLayerByName($name, $layer['childs']);
                if ($recursiveResult) {
                    foreach($recursiveResult as &$r) {
                        $return[] = &$r;
                    }
                }
            }
        }
        return $return;
    }
}