<?php
/**
 * con4gis - the gis-kit
 *
 * @version   php 5
 * @package   con4gis
 * @author    con4gis contributors (see "authors.txt")
 * @license   GNU/LGPL http://opensource.org/licenses/lgpl-3.0.html
 * @copyright Küstenschmiede GmbH Software & Design 2011 - 2016.
 * @link      https://www.kuestenschmiede.de
 */

namespace con4gis\MapsBundle\Classes;


use con4gis\MapsBundle\Entity\BaseLayer;
use Doctrine\ORM\EntityManager;

class MapTileLoader
{
    /**
     * Requests the map tile for the given extent and returns it.
     * @param $baseLayerId
     * @param $z
     * @param $x
     * @param $y
     * @param EntityManager $em
     * @return string
     */
    public function loadTileForExtent($baseLayerId, $z, $x, $y, EntityManager $em)
    {
        // TODO replace with caching
//        if(\Session::getInstance()->get('baseLayerId'.$intBaseLayerId))
//        {
//            $baseLayer = \Session::getInstance()->get('baseLayerId'.$intBaseLayerId);
//        }
//        else{
//            $baseLayer = C4gMapBaselayersModel::findById($intBaseLayerId);
//            \Session::getInstance()->set('baseLayerId'.$intBaseLayerId,$baseLayer->row());
//            $baseLayer = $baseLayer->row();
//        }
        $baseLayer = $em->getRepository('\con4gis\MapsBundle\Entity\BaseLayer')
            ->findOneBy(['id' => $baseLayerId]);
        if (!($baseLayer && $baseLayer instanceof BaseLayer)) {
            // no baselayer found, return HTTP Error code!
        }
        switch($baseLayer->getProvider())
        {
            case "mapbox":
                if($baseLayer->getApiKey() && $baseLayer->getAppId() && $baseLayer->getMapboxType())
                {
                    if($baseLayer->getMapboxType() === 'Mapbox')
                    {
                        $url = 'https://api.mapbox.com/styles/v1/' . $baseLayer->getAppId() . '/tiles/'.$z.'/'.$x.'/'.$y.'?access_token=' . $baseLayer->getApiKey();
                    }
                    else
                    {
                        $url = "https://api.tiles.mapbox.com/v4/". $baseLayer->getAppId() . '/tiles/'.$z.'/'.$x.'/'.$y.'?access_token=' . $baseLayer->getApiKey();
                    }
                }
                else{
                    return "wrong Mapbox configuration";
                }
                break;
//            case "bing":
//                if($baseLayer->apiKey && $baseLayer->style)
//                {
//
//                }
//                break;
            case "here":
                if($baseLayer->getApiKey() && $baseLayer->getAppId() && $baseLayer->getHereType()){
                    // TODO Hier kann man sicher einige Fälle zusammenfassen!
                    if ($baseLayer->getHereType() == 'normal') {
                        $url = 'https://{1-4}.base.maps.cit.api.here.com/maptile/2.1/maptile/newest/normal.day/'.$z.'/'.$x.'/'.$y.'/256/png' .
                            '?app_id='.$baseLayer->getAppId().'&app_code='.$baseLayer->getApiKey();
                    }
                    if ($baseLayer->getHereType() == 'transit') {
                        $url = 'https://{1-4}.base.maps.cit.api.here.com/maptile/2.1/maptile/newest/normal.day.transit/'.$z.'/'.$x.'/'.$y.'/256/png' .
                            '?app_id='.$baseLayer->getAppId().'&app_code='.$baseLayer->getApiKey();
                    }
                    if ($baseLayer->getHereType() == 'pedestrian') {
                        $url = 'https://{1-4}.base.maps.cit.api.here.com/maptile/2.1/maptile/newest/pedestrian.day/'.$z.'/'.$x.'/'.$y.'/256/png' .
                            '?app_id='.$baseLayer->getAppId().'&app_code='.$baseLayer->getApiKey();
                    }
                    if ($baseLayer->getHereType() == 'terrain') {
                        $url = 'https://{1-4}.aerial.maps.cit.api.here.com/maptile/2.1/maptile/newest/terrain.day/'.$z.'/'.$x.'/'.$y.'/256/png' .
                            '?app_id='.$baseLayer->getAppId().'&app_code='.$baseLayer->getApiKey();
                    }
                    if ($baseLayer->getHereType() == 'satellite') {
                        $url = 'https://{1-4}.aerial.maps.cit.api.here.com/maptile/2.1/maptile/newest/satellite.day/'.$z.'/'.$x.'/'.$y.'/256/png' .
                            '?app_id='.$baseLayer->getAppId().'&app_code='.$baseLayer->getApiKey();
                    }
                    if ($baseLayer->getHereType() == 'hybrid') {
                        $url = 'https://{1-4}.aerial.maps.cit.api.here.com/maptile/2.1/maptile/newest/hybrid.day/'.$z.'/'.$x.'/'.$y.'/256/png' .
                            '?app_id='.$baseLayer->getAppId().'&app_code='.$baseLayer->getApiKey();
                    }
                }
                break;
            case "thunder":
                if($baseLayer->getApiKey() && $baseLayer->getThunderforestType()){
                    $url = 'https://tile.thunderforest.com/' . $baseLayer->getThunderforestType() . '/'.$z.'/'.$x.'/'.$y.'.png?apikey=' . $baseLayer->getApiKey();
                }
                else
                {
                    return "wrong Thunderforest configuration";
                }
                break;

        }
        $REQUEST = new \Request();
        if ($_SERVER['HTTP_REFERER']) {
            $REQUEST->setHeader('Referer', $_SERVER['HTTP_REFERER']);
        }
        if ($_SERVER['HTTP_USER_AGENT']) {
            $REQUEST->setHeader('User-Agent', $_SERVER['HTTP_USER_AGENT']);
        }
        if($url)
        {
            $REQUEST->send($url);
            return $REQUEST->response;
        }
        else{
            return "wrong baselayer configuration";
        }
    }
}