<?php
/**
 * con4gis - the gis-kit
 *
 * @version   php 5
 * @package   con4gis
 * @author    con4gis contributors (see "authors.txt")
 * @license   GNU/LGPL http://opensource.org/licenses/lgpl-3.0.html
 * @copyright Küstenschmiede GmbH Software & Design 2011 - 2016.
 * @link      https://www.kuestenschmiede.de
 */

namespace con4gis\MapsBundle\Classes;

use con4gis\MapsBundle\Entity\Overlay;

class OverlayParser
{
    /**
     * Creates a JSON representation like it is used by the JS frontend of the passed Overlay entity.
     * @param Overlay $overlay
     * @return array
     */
    public function parseOverlay(Overlay $overlay)
    {
        $stringClass = $GLOBALS['con4gis']['stringClass'];
        $arrOverlayData = array();

        $arrOverlayData['id'] = $overlay->getId();
        $arrOverlayData['pid'] = $overlay->getPid();
        $arrOverlayData['name'] =  \Contao\Controller::replaceInsertTags($stringClass::decodeEntities($overlay->getName()));

        $arrOverlayData['provider'] = $overlay->getProvider();

        switch ($overlay->getProvider())
        {
            case "custom":
                /* this values work for the js-code */
                $arrOverlayData['provider'] = "osm";
                $arrOverlayData['style'] = "osm_custom";

                $arrOverlayData['urls'] = array();

                for ($intI=1; $intI<5; $intI++)
                {
                    $colName = "url" . $intI;
                    if ($overlay->$colName)
                    {
                        $arrOverlayData['urls'][] = str_replace("$", "", $overlay->$colName);
                    }
                }
                break;
            case 'wms':
                $arrOverlayData['url'] = $overlay->getWmsUrl();

                $arrOverlayData['params'] = array();
                $arrOverlayData['params']['layers'] = $overlay->getWmsParamsLayers();
                $arrOverlayData['params']['version'] = $overlay->getWmsParamsVersion();
                $arrOverlayData['params']['format'] = $overlay->getWmsParamsFormat();
                $arrOverlayData['params']['transparent'] = $overlay->getWmsParamsTransparent() ? true : false;
                $arrOverlayData['params']['srs'] = $overlay->getWmsParamsSrs();
                $arrOverlayData['gutter'] = $overlay->getWmsGutter();
                break;
            case 'owm':
                $arrOverlayData['url'] = 'http://maps.owm.io:'.$overlay->getApiPort().'/';

                $arrOverlayData['app_id'] = $overlay->getAppId();
                $arrOverlayData['api_key'] = $overlay->getApiKey();
                break;
        }
        $arrOverlayData['opacity']     = $overlay->getOpacity();
        $arrOverlayData['attribution'] = $overlay->getAttribution();


        return $arrOverlayData;
    }
}