<?php
/**
 * con4gis - the gis-kit
 *
 * @version   php 5
 * @package   con4gis
 * @author    con4gis contributors (see "authors.txt")
 * @license   GNU/LGPL http://opensource.org/licenses/lgpl-3.0.html
 * @copyright Küstenschmiede GmbH Software & Design 2011 - 2016.
 * @link      https://www.kuestenschmiede.de
 */

namespace con4gis\MapsBundle\Controller;


use con4gis\CoreBundle\Controller\BaseController;
use con4gis\MapsBundle\Classes\BaseLayerParser;
use con4gis\MapsBundle\Classes\MapTileLoader;
use con4gis\MapsBundle\Classes\OverlayParser;
use con4gis\MapsBundle\Entity\BaseLayer;
use con4gis\MapsBundle\Entity\Overlay;
use con4gis\MapsBundle\Resources\contao\models\C4gMapOverlaysModel;
use con4gis\MapsBundle\Resources\contao\models\C4gMapProfilesModel;
use Doctrine\ORM\EntityManager;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

class BaseLayerController extends BaseController
{
    /**
     * @var BaseLayerParser
     */
    private $layerParser = null;

    /**
     * @var OverlayParser
     */
    private $overlayParser = null;

    /**
     * @var MapTileLoader
     */
    private $tileLoader = null;

    protected function initialize()
    {
        parent::initialize();
        $this->layerParser = new BaseLayerParser();
        $this->overlayParser = new OverlayParser();
        $this->tileLoader = new MapTileLoader();
    }

    public function baseLayerAction(Request $request, $profileId)
    {
        $this->initialize();
        $response = new JsonResponse();
        $this->checkForCacheSettings('baseLayerService');
        if (self::$useCache) {
            $this->checkAndStoreCachedData($request);
        }
        if (!self::$outputFromCache) {
            $this->responseData = $this->getBaseLayers($profileId);
            if (self::$useCache) {
                $this->storeDataInCache($request);
            }
        }
        $response->setData($this->responseData);
        return $response;
    }
    public function baseLayerTileAction(Request $request, $baseLayerId, $z, $x, $y)
    {
        $this->responseData = $this->tileLoader->loadTileForExtent($baseLayerId, $z, $x, $y, $this->entityManager);
        $response = new Response($this->responseData);
        $response->headers->set('Content-Type', 'application/png');
        return $response;
    }

    private function getBaseLayers($profileId)
    {
        $arrConfig = array();
        $blnProfileBaselayerFilter = false;
        $mapsProfileModel = C4gMapProfilesModel::findById($profileId);

        if ($mapsProfileModel !== null)
        {
            if ($mapsProfileModel->baselayers)
            {
                $blnProfileBaselayerFilter = true;
                $arrProfileBaselayerFilter = deserialize($mapsProfileModel->baselayers, true);
            }
        }

        $arrLayers = $this->getBaseLayerList($blnProfileBaselayerFilter ? $arrProfileBaselayerFilter : []);

        $arrConfig['countAll'] = sizeof($arrLayers);

        return array('config' => $arrConfig, 'baselayer' => $arrLayers);
    }

    private function getBaseLayerList($arrFilter)
    {
        $arrBaseLayerData = array();
        // ToDo get EntityManager via Dependency Injection
        $em = $this->container->get('doctrine.orm.entity_manager');
        $baseLayerRepo = $em->getRepository('\con4gis\MapsBundle\Entity\BaseLayer');
        $overlayRepo = $em->getRepository('\con4gis\MapsBundle\Entity\Overlay');
        if (count($arrFilter) > 0) {
            // use baselayer selection from profile
            $baseLayers = $baseLayerRepo->findBy(['id' => $arrFilter]);
        } else {
            // get all baselayers
            $baseLayers = $baseLayerRepo->findAll();
        }
        /** @var BaseLayer $baseLayer */
        foreach ($baseLayers as $baseLayer) {
            if ($baseLayer->getProtectBaselayer()) {
                if (\FrontendUser::getInstance()->authenticate() && !empty($baseLayer->getPermittedGroups())) {
                    // get groups data from tl_member table
                    $userGroups = \FrontendUser::getInstance()->getData()['groups'];
                    $permittedGroups = deserialize($baseLayer->getPermittedGroups());
                    if (sizeof(array_intersect($userGroups, $permittedGroups)) < 1) {
                        continue;
                    }
                }
                $layerData = $this->layerParser->parseBaseLayer($baseLayer);
                $overlays = $overlayRepo->findBy(['pid' => $baseLayer->getId()]);
                if (count($overlays) > 0) {
                    $layerData['hasOverlays'] = true;
                    $layerData['overlays'] = array();
                    /** @var Overlay $overlay */
                    foreach ($overlays as $overlay) {
                        $layerData['overlays'][] = $this->overlayParser->parseOverlay($overlay);
                    }
                }
                $arrBaseLayerData[] = $layerData;
            }
        }
        return $arrBaseLayerData;
    }
}